@@ -23,6 +23,7 @@ try:
 except ImportError:
     import mock
 from docker import errors as docker_error
+from docker.types import Ulimit
 from oslotest import base
 
 this_dir = os.path.dirname(sys.modules[__name__].__file__)
@@ -44,7 +45,7 @@ class ModuleArgsTest(base.BaseTestCase):
                 choices=['compare_container', 'compare_image', 'create_volume',
                          'get_container_env', 'get_container_state',
                          'pull_image', 'recreate_or_restart_container',
-                         'remove_container', 'remove_volume',
+                         'remove_container', 'remove_image', 'remove_volume',
                          'restart_container', 'start_container',
                          'stop_container']),
             api_version=dict(required=False, type='str', default='auto'),
@@ -99,6 +100,7 @@ class ModuleArgsTest(base.BaseTestCase):
             ['action', 'get_container_state', ['name']],
             ['action', 'recreate_or_restart_container', ['name']],
             ['action', 'remove_container', ['name']],
+            ['action', 'remove_image', ['image']],
             ['action', 'remove_volume', ['name']],
             ['action', 'restart_container', ['name']],
             ['action', 'stop_container', ['name']]
@@ -700,6 +702,61 @@ class TestImage(base.BaseTestCase):
             msg="Unknown error message: unexpected error",
             failed=True)
 
+    def test_remove_image(self):
+        self.dw = get_DockerWorker(
+            {'image': 'myregistrydomain.com:5000/ubuntu:16.04',
+             'action': 'remove_image'})
+        self.dw.dc.images.return_value = self.fake_data['images']
+
+        self.dw.remove_image()
+        self.assertTrue(self.dw.changed)
+        self.dw.dc.remove_image.assert_called_once_with(
+            image='myregistrydomain.com:5000/ubuntu:16.04')
+
+    def test_remove_image_not_exists(self):
+        self.dw = get_DockerWorker(
+            {'image': 'myregistrydomain.com:5000/non_existing:16.04',
+             'action': 'remove_image'})
+        self.dw.dc.images.return_value = self.fake_data['images']
+
+        self.dw.remove_image()
+        self.assertFalse(self.dw.changed)
+
+    def test_remove_image_exception_409(self):
+        resp = mock.MagicMock()
+        resp.status_code = 409
+        docker_except = docker_error.APIError('test error', resp)
+        self.dw = get_DockerWorker(
+            {'image': 'myregistrydomain.com:5000/ubuntu:16.04',
+             'action': 'remove_image'})
+        self.dw.dc.images.return_value = self.fake_data['images']
+        self.dw.dc.remove_image.side_effect = docker_except
+
+        self.assertRaises(docker_error.APIError, self.dw.remove_image)
+        self.assertTrue(self.dw.changed)
+        self.dw.module.fail_json.assert_called_once_with(
+            failed=True,
+            msg=("Image 'myregistrydomain.com:5000/ubuntu:16.04' "
+                 "is currently in-use")
+        )
+
+    def test_remove_image_exception_500(self):
+        resp = mock.MagicMock()
+        resp.status_code = 500
+        docker_except = docker_error.APIError('test error', resp)
+        self.dw = get_DockerWorker(
+            {'image': 'myregistrydomain.com:5000/ubuntu:16.04',
+             'action': 'remove_image'})
+        self.dw.dc.images.return_value = self.fake_data['images']
+        self.dw.dc.remove_image.side_effect = docker_except
+
+        self.assertRaises(docker_error.APIError, self.dw.remove_image)
+        self.assertTrue(self.dw.changed)
+        self.dw.module.fail_json.assert_called_once_with(
+            failed=True,
+            msg=("Server error")
+        )
+
 
 class TestVolume(base.BaseTestCase):
 
@@ -911,7 +968,8 @@ class TestAttrComp(base.BaseTestCase):
         container_info['HostConfig'] = {
             'CpuPeriod': 0, 'KernelMemory': 0, 'Memory': 0, 'CpuQuota': 0,
             'CpusetCpus': '', 'CpuShares': 0, 'BlkioWeight': 0,
-            'CpusetMems': '', 'MemorySwap': 0, 'MemoryReservation': 0}
+            'CpusetMems': '', 'MemorySwap': 0, 'MemoryReservation': 0,
+            'Ulimits': []}
         self.dw = get_DockerWorker(self.fake_data['params'])
         self.assertTrue(self.dw.compare_dimensions(container_info))
 
@@ -922,7 +980,8 @@ class TestAttrComp(base.BaseTestCase):
         container_info['HostConfig'] = {
             'CpuPeriod': 0, 'KernelMemory': 0, 'Memory': 0, 'CpuQuota': 0,
             'CpusetCpus': '', 'CpuShares': 0, 'BlkioWeight': 10,
-            'CpusetMems': '', 'MemorySwap': 0, 'MemoryReservation': 0}
+            'CpusetMems': '', 'MemorySwap': 0, 'MemoryReservation': 0,
+            'Ulimits': []}
         self.dw = get_DockerWorker(self.fake_data['params'])
         self.assertFalse(self.dw.compare_dimensions(container_info))
 
@@ -933,7 +992,8 @@ class TestAttrComp(base.BaseTestCase):
         container_info['HostConfig'] = {
             'CpuPeriod': 0, 'KernelMemory': 0, 'Memory': 0, 'CpuQuota': 0,
             'CpusetCpus': '', 'CpuShares': 0, 'BlkioWeight': 0,
-            'CpusetMems': '', 'MemorySwap': 0, 'MemoryReservation': 0}
+            'CpusetMems': '', 'MemorySwap': 0, 'MemoryReservation': 0,
+            'Ulimits': []}
         self.dw = get_DockerWorker(self.fake_data['params'])
         self.dw.compare_dimensions(container_info)
         self.dw.module.exit_json.assert_called_once_with(
@@ -946,7 +1006,8 @@ class TestAttrComp(base.BaseTestCase):
         container_info['HostConfig'] = {
             'CpuPeriod': 0, 'KernelMemory': 0, 'Memory': 0, 'CpuQuota': 0,
             'CpusetCpus': '1', 'CpuShares': 0, 'BlkioWeight': 0,
-            'CpusetMems': '', 'MemorySwap': 0, 'MemoryReservation': 0}
+            'CpusetMems': '', 'MemorySwap': 0, 'MemoryReservation': 0,
+            'Ulimits': []}
         self.dw = get_DockerWorker(self.fake_data['params'])
         self.assertTrue(self.dw.compare_dimensions(container_info))
 
@@ -961,7 +1022,8 @@ class TestAttrComp(base.BaseTestCase):
         container_info['HostConfig'] = {
             'CpuPeriod': 0, 'KernelMemory': 0, 'Memory': 10, 'CpuQuota': 0,
             'CpusetCpus': '', 'CpuShares': 0, 'BlkioWeight': 0,
-            'CpusetMems': '', 'MemorySwap': 0, 'MemoryReservation': 10}
+            'CpusetMems': '', 'MemorySwap': 0, 'MemoryReservation': 10,
+            'Ulimits': []}
         self.dw = get_DockerWorker(self.fake_data['params'])
         self.assertTrue(self.dw.compare_dimensions(container_info))
 
@@ -976,7 +1038,8 @@ class TestAttrComp(base.BaseTestCase):
         container_info['HostConfig'] = {
             'CpuPeriod': 0, 'KernelMemory': 0, 'Memory': 0, 'CpuQuota': 0,
             'CpusetCpus': '', 'CpuShares': 0, 'BlkioWeight': 0,
-            'CpusetMems': '', 'MemorySwap': 0, 'MemoryReservation': 0}
+            'CpusetMems': '', 'MemorySwap': 0, 'MemoryReservation': 0,
+            'Ulimits': []}
         self.dw = get_DockerWorker(self.fake_data['params'])
         self.assertFalse(self.dw.compare_dimensions(container_info))
 
@@ -984,3 +1047,29 @@ class TestAttrComp(base.BaseTestCase):
         container_info = {'State': dict(Status='running')}
         self.dw = get_DockerWorker({'state': 'exited'})
         self.assertTrue(self.dw.compare_container_state(container_info))
+
+    def test_compare_ulimits_pos(self):
+        self.fake_data['params']['dimensions'] = {
+            'ulimits': {'nofile': {'soft': 131072, 'hard': 131072}}}
+        container_info = dict()
+        container_info['HostConfig'] = {
+            'CpuPeriod': 0, 'KernelMemory': 0, 'Memory': 0, 'CpuQuota': 0,
+            'CpusetCpus': '', 'CpuShares': 0, 'BlkioWeight': 0,
+            'CpusetMems': '', 'MemorySwap': 0, 'MemoryReservation': 0,
+            'Ulimits': []}
+        self.dw = get_DockerWorker(self.fake_data['params'])
+        self.assertTrue(self.dw.compare_dimensions(container_info))
+
+    def test_compare_ulimits_neg(self):
+        self.fake_data['params']['dimensions'] = {
+            'ulimits': {'nofile': {'soft': 131072, 'hard': 131072}}}
+        ulimits_nofile = Ulimit(name='nofile',
+                                soft=131072, hard=131072)
+        container_info = dict()
+        container_info['HostConfig'] = {
+            'CpuPeriod': 0, 'KernelMemory': 0, 'Memory': 0, 'CpuQuota': 0,
+            'CpusetCpus': '', 'CpuShares': 0, 'BlkioWeight': 0,
+            'CpusetMems': '', 'MemorySwap': 0, 'MemoryReservation': 0,
+            'Ulimits': [ulimits_nofile]}
+        self.dw = get_DockerWorker(self.fake_data['params'])
+        self.assertFalse(self.dw.compare_dimensions(container_info))
